'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* global window, navigator */

var _getIt = require('get-it');

var _getIt2 = _interopRequireDefault(_getIt);

var _jsonResponse = require('get-it/lib/middleware/jsonResponse');

var _jsonResponse2 = _interopRequireDefault(_jsonResponse);

var _promise = require('get-it/lib/middleware/promise');

var _promise2 = _interopRequireDefault(_promise);

var _minimal = require('@sanity/observable/minimal');

var _minimal2 = _interopRequireDefault(_minimal);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var EventSource = typeof window !== 'undefined' && window.EventSource ? window.EventSource // Native browser EventSource
: require('@sanity/eventsource'); // Node.js, IE etc

function parseEvent(event) {
  try {
    return event.data && JSON.parse(event.data) || {};
  } catch (err) {
    return err;
  }
}

var Reflector = function () {
  function Reflector(sanityClient) {
    _classCallCheck(this, Reflector);

    this.sanityClient = sanityClient;
    this.request = (0, _getIt2.default)([(0, _jsonResponse2.default)(), (0, _promise2.default)()]);
  }

  _createClass(Reflector, [{
    key: 'listen',
    value: function listen(channel) {
      var _sanityClient$clientC = this.sanityClient.clientConfig,
          token = _sanityClient$clientC.token,
          withCredentials = _sanityClient$clientC.withCredentials;


      var esOptions = {};
      if (token || withCredentials) {
        esOptions.withCredentials = true;
      }

      if (token) {
        esOptions.headers = {
          Authorization: 'Bearer ' + token
        };
      }

      var url = this.sanityClient.getUrl('presence/listen/' + channel);
      return new _minimal2.default(function (observer) {
        var es = new EventSource(url, esOptions);
        es.addEventListener('message', onMessage, false);

        function onMessage(evt) {
          var event = parseEvent(evt);
          return event instanceof Error ? observer.error(event) : observer.next(event);
        }

        function unsubscribe() {
          es.removeEventListener('message', onMessage, false);
          es.close();
        }

        return unsubscribe;
      });
    }
  }, {
    key: 'send',
    value: function send(channel, message) {
      var url = this.sanityClient.getUrl('presence/send/' + channel);

      var headers = {};
      var token = this.sanityClient.clientConfig.token;

      if (token) {
        headers = {
          Authorization: 'Bearer ' + token
        };
      }

      return this.request({
        url: url,
        method: 'POST',
        headers: headers,
        body: JSON.stringify(message)
      });
    }

    // Sends a message using the beacon api which in some browsers lets us send a little bit of
    // data while the window is closing. Returns true if the message was successfully submitted,
    // false if it failed or if status is unknown.

  }, {
    key: 'sendBeacon',
    value: function sendBeacon(channel, message) {
      if (typeof navigator == 'undefined' || typeof navigator.sendBeacon != 'function') {
        // If sendBeacon is not supported, just try to send it the old fashioned way
        this.send(channel, message);
        return false;
      }
      var url = this.sanityClient.getUrl('presence/send/' + channel);
      return navigator.sendBeacon(url, JSON.stringify(message));
    }

    // Create a connection to a specific reflector channel

  }, {
    key: 'connect',
    value: function connect(channel) {
      var _this = this;

      return {
        listen: function listen() {
          return _this.listen(channel);
        },
        send: function send(message) {
          return _this.send(channel, message);
        },
        sendBeacon: function sendBeacon(message) {
          return _this.sendBeacon(channel, message);
        }
      };
    }
  }]);

  return Reflector;
}();

exports.default = Reflector;